@@ -33,6 +33,7 @@ gint backend_init_int (struct backend *, const gchar *);
 gboolean backend_check_int (struct backend *);
 const gchar *backend_name ();
 const gchar *backend_version ();
+void backend_fill_devices_array (GArray *);
 
 //Identity Request Universal Sysex message
 static const guint8 BE_MIDI_IDENTITY_REQUEST[] =
@@ -272,7 +273,7 @@ backend_tx_and_rx_sysex (struct backend *backend, GByteArray * tx_msg,
 gchar *
 backend_get_fs_ext (struct backend *backend, const struct fs_operations *ops)
 {
-  gchar *ext = malloc (LABEL_MAX);
+  gchar *ext = g_malloc (LABEL_MAX);
   snprintf (ext, LABEL_MAX, "%s", ops->type_ext);
   return ext;
 }
@@ -305,18 +306,18 @@ backend_program_change (struct backend *backend, guint8 channel,
 }
 
 gint
-backend_send_controller (struct backend *backend, guint8 channel,
-			 guint8 controller, guint8 value)
+backend_send_3_byte_message (struct backend *backend, guint8 msg_type,
+			     guint8 channel, guint8 d1, guint8 d2)
 {
   ssize_t size;
   guint8 msg[3];
 
-  msg[0] = 0xb0 | (channel & 0xf);
-  msg[1] = controller & 0x7f;
-  msg[2] = value & 0x7f;
+  msg[0] = msg_type | (channel & 0xf);
+  msg[1] = d1 & 0x7f;
+  msg[2] = d2 & 0x7f;
 
-  debug_print (1, "Sending MIDI controller %d with value %d...\n", msg[1],
-	       msg[2]);
+  debug_print (1, "Sending MIDI message: status %08x; data %d, %d...\n",
+	       msg[0], msg[1], msg[2]);
 
   if ((size = backend_tx_raw (backend, msg, 3)) < 0)
     {
@@ -325,6 +326,29 @@ backend_send_controller (struct backend *backend, guint8 channel,
   return 0;
 }
 
+gint
+backend_send_controller (struct backend *backend, guint8 channel,
+			 guint8 controller, guint8 value)
+{
+  return backend_send_3_byte_message (backend, 0xb0, channel, controller,
+				      value);
+}
+
+gint
+backend_send_note_on (struct backend *backend, guint8 channel,
+		      guint8 note, guint8 velocity)
+{
+  return backend_send_3_byte_message (backend, 0x90, channel, note, velocity);
+}
+
+gint
+backend_send_note_off (struct backend *backend, guint8 channel,
+		       guint8 note, guint8 velocity)
+{
+  return backend_send_3_byte_message (backend, 0x80, channel, note, velocity);
+}
+
+
 gint
 backend_send_rpn (struct backend *backend, guint8 channel,
 		  guint8 controller_msb, guint8 controller_lsb,
@@ -432,7 +456,7 @@ backend_rx_raw_loop (struct backend *backend, struct sysex_transfer *transfer)
 					    backend->rx_len);
 	  debug_print (4, "Internal buffer data (%zd): %s\n", backend->rx_len,
 		       text);
-	  free (text);
+	  g_free (text);
 	  return -ETIMEDOUT;
 	}
 
@@ -460,7 +484,7 @@ backend_rx_raw_loop (struct backend *backend, struct sysex_transfer *transfer)
 	      gchar *text = debug_get_hex_data (debug_level, tmp, rx_len);
 	      debug_print (4, "Skipping non SysEx data (%zd): %s\n", rx_len,
 			   text);
-	      free (text);
+	      g_free (text);
 	    }
 
 	  tmp_msg++;
@@ -498,7 +522,7 @@ backend_rx_raw_loop (struct backend *backend, struct sysex_transfer *transfer)
     {
       text = debug_get_hex_data (debug_level, data, rx_len);
       debug_print (3, "Queued data (%zu): %s\n", rx_len, text);
-      free (text);
+      g_free (text);
     }
 
   return rx_len;
@@ -581,7 +605,7 @@ backend_rx_sysex (struct backend *backend, struct sysex_transfer *transfer)
 						i);
 	      debug_print (4, "Skipping non SysEx data in buffer (%d): %s\n",
 			   i, text);
-	      free (text);
+	      g_free (text);
 	    }
 	  debug_print (3, "Copying %d bytes...\n", len - i);
 	  g_byte_array_append (transfer->raw, b, len - i);
@@ -608,7 +632,7 @@ backend_rx_sysex (struct backend *backend, struct sysex_transfer *transfer)
 				    transfer->raw->len);
 	      debug_print (4, "Queued data (%d): %s\n",
 			   transfer->raw->len, text);
-	      free (text);
+	      g_free (text);
 	    }
 	}
       else
@@ -640,7 +664,7 @@ end:
 					    transfer->raw->len);
 	  debug_print (2, "Raw message received (%d): %s\n",
 		       transfer->raw->len, text);
-	  free (text);
+	  g_free (text);
 	}
 
     }
@@ -673,3 +697,20 @@ path_type_from_backend (struct backend *backend)
   return (!backend || backend->type == BE_TYPE_SYSTEM) ? PATH_SYSTEM :
     PATH_INTERNAL;
 }
+
+GArray *
+backend_get_devices ()
+{
+  struct backend_device *backend_device;
+  GArray *devices =
+    g_array_new (FALSE, FALSE, sizeof (struct backend_device));
+
+  backend_device = g_malloc (sizeof (struct backend_device));
+  backend_device->type = BE_TYPE_SYSTEM;
+  snprintf (backend_device->id, LABEL_MAX, "%s", BE_SYSTEM_ID);
+  snprintf (backend_device->name, LABEL_MAX, "%s", g_get_host_name ());
+  g_array_append_vals (devices, backend_device, 1);
+
+  backend_fill_devices_array (devices);
+  return devices;
+}
