@@ -44,12 +44,12 @@
 #define BE_FILE_ICON_SEQ "elektroid-sequence-symbolic"
 #define BE_FILE_ICON_PRJ "elektroid-project-symbolic"
 #define BE_FILE_ICON_SND "elektroid-sound-symbolic"
+#define BE_FILE_ICON_GENERIC "text-x-generic-symbolic"
 
 #define BE_DUMP_TIMEOUT 5000	//With and E-Mu ESI-2000 it takes this more than 3 seconds to receive to receive some packets after the process has started.
 #define BE_REST_TIME_US 50000
 #define BE_SYSEX_TIMEOUT_MS 5000
 #define BE_SYSEX_TIMEOUT_GUESS_MS 1000	//When the request is not implemented, 5 s is too much.
-#define BE_SAMPLE_ID_NAME_SEPARATOR ":"
 
 #define BE_COMPANY_LEN 3
 #define BE_FAMILY_LEN 2
@@ -60,7 +60,7 @@
 
 struct backend_storage_stats
 {
-  const gchar *name;
+  gchar name[LABEL_MAX];
   guint64 bsize;
   guint64 bfree;
 };
@@ -68,7 +68,8 @@ struct backend_storage_stats
 typedef void (*t_destroy_data) (struct backend *);
 
 typedef gint (*t_get_storage_stats) (struct backend *, gint,
-				     struct backend_storage_stats *);
+				     struct backend_storage_stats *,
+				     const gchar *);
 
 struct backend_midi_info
 {
@@ -118,8 +119,9 @@ struct backend
   t_get_storage_stats get_storage_stats;	//This function is device function, not a filesystem function. Several filesystems might share the same memory.
 };
 
-struct backend_system_device
+struct backend_device
 {
+  enum backend_type type;
   gchar name[LABEL_MAX];
   gchar id[LABEL_MAX];
 };
@@ -153,7 +155,7 @@ void backend_enable_cache (struct backend *);
 
 void backend_disable_cache (struct backend *);
 
-GArray *backend_get_system_devices ();
+GArray *backend_get_devices ();
 
 const struct fs_operations *backend_get_fs_operations (struct backend *, gint,
 						       const char *);
@@ -171,6 +173,12 @@ gint backend_program_change (struct backend *, guint8, guint8);
 gint backend_send_controller (struct backend *backend, guint8 channel,
 			      guint8 controller, guint8 value);
 
+gint backend_send_note_on (struct backend *backend, guint8 channel,
+			   guint8 note, guint8 velocity);
+
+gint backend_send_note_off (struct backend *backend, guint8 channel,
+			    guint8 note, guint8 velocity);
+
 gint backend_send_rpn (struct backend *backend, guint8 channel,
 		       guint8 controller_msb, guint8 controller_lsb,
 		       guint8 value_msb, guint8 value_lsb);
@@ -185,4 +193,6 @@ const gchar *backend_strerror (struct backend *backend, gint error);
 
 enum path_type path_type_from_backend (struct backend *);
 
+const gchar *backend_name ();
+
 #endif
