@@ -49,13 +49,13 @@ backend_destroy_int (struct backend *backend)
 
   if (backend->buffer)
     {
-      free (backend->buffer);
+      g_free (backend->buffer);
       backend->buffer = NULL;
     }
 
   if (backend->pfds)
     {
-      free (backend->pfds);
+      g_free (backend->pfds);
       backend->pfds = NULL;
     }
 }
@@ -95,7 +95,7 @@ backend_init_int (struct backend *backend, const gchar * id)
     }
 
   backend->npfds = snd_rawmidi_poll_descriptors_count (backend->inputp);
-  backend->pfds = malloc (backend->npfds * sizeof (struct pollfd));
+  backend->pfds = g_malloc (backend->npfds * sizeof (struct pollfd));
 
   snd_rawmidi_poll_descriptors (backend->inputp, backend->pfds,
 				backend->npfds);
@@ -204,7 +204,7 @@ backend_tx_sysex_internal (struct backend *backend,
 					transfer->raw->len);
       debug_print (2, "Raw message sent (%d): %s\n", transfer->raw->len,
 		   text);
-      free (text);
+      g_free (text);
     }
 
   if (update)
@@ -296,7 +296,7 @@ backend_get_system_subdevices (snd_ctl_t * ctl, int card, int device,
   int subs, subs_in, subs_out;
   int sub;
   int err;
-  struct backend_system_device *backend_system_device;
+  struct backend_device *backend_device;
 
   snd_rawmidi_info_alloca (&info);
   snd_rawmidi_info_set_device (info, device);
@@ -362,14 +362,15 @@ backend_get_system_subdevices (snd_ctl_t * ctl, int card, int device,
 
       debug_print (1, "Adding hw:%d (name '%s', subname '%s')...\n", card,
 		   name, sub_name);
-      backend_system_device = malloc (sizeof (struct backend_system_device));
-      snprintf (backend_system_device->id, LABEL_MAX, BE_DEVICE_NAME, card,
-		device, sub);
-      snprintf (backend_system_device->name, LABEL_MAX,
-		BE_DEVICE_NAME ": %s%s%s", card, device, sub, name,
-		strlen (sub_name) ? ", " : "", sub_name);
+      backend_device = g_malloc (sizeof (struct backend_device));
+      backend_device->type = BE_TYPE_MIDI;
+      snprintf (backend_device->id, LABEL_MAX, BE_DEVICE_NAME, card, device,
+		sub);
+      snprintf (backend_device->name, LABEL_MAX, BE_DEVICE_NAME ": %s%s%s",
+		card, device, sub, name, strlen (sub_name) ? ", " : "",
+		sub_name);
 
-      g_array_append_vals (devices, backend_system_device, 1);
+      g_array_append_vals (devices, backend_device, 1);
     }
 }
 
@@ -401,13 +402,10 @@ backend_fill_card_devices (gint card, GArray * devices)
   snd_ctl_close (ctl);
 }
 
-GArray *
-backend_get_system_devices ()
+void
+backend_fill_devices_array (GArray * devices)
 {
   gint card, err;
-  GArray *devices =
-    g_array_new (FALSE, FALSE, sizeof (struct backend_system_device));
-
   card = -1;
   while (!(err = snd_card_next (&card)) && card >= 0)
     {
@@ -417,8 +415,6 @@ backend_get_system_devices ()
     {
       error_print ("Cannot determine card number: %s\n", snd_strerror (err));
     }
-
-  return devices;
 }
 
 const gchar *
