@@ -156,7 +156,7 @@ backend_tx_sysex_internal (struct backend *backend,
 					transfer->raw->len);
       debug_print (2, "Raw message sent (%d): %s\n", transfer->raw->len,
 		   text);
-      free (text);
+      g_free (text);
     }
 
   if (update)
@@ -215,8 +215,8 @@ backend_check_int (struct backend *backend)
   return backend->inputp && backend->outputp;
 }
 
-GArray *
-backend_get_system_devices ()
+void
+backend_fill_devices_array (GArray * devices)
 {
   struct RtMidiWrapper *inputp;
   struct RtMidiWrapper *outputp;
@@ -224,13 +224,11 @@ backend_get_system_devices ()
   gchar iportname[LABEL_MAX];
   gchar oportname[LABEL_MAX];
   gint iportnamelen, oportnamelen;
-  struct backend_system_device *backend_system_device;
-  GArray *devices = g_array_new (FALSE, FALSE,
-				 sizeof (struct backend_system_device));
+  struct backend_device *backend_device;
 
   if (!(inputp = rtmidi_in_create_default ()))
     {
-      goto end;
+      return;
     }
 
   if (!(outputp = rtmidi_out_create_default ()))
@@ -265,23 +263,21 @@ backend_get_system_devices ()
 #if defined(__linux__)
 	  if (!strcmp (iportname, oportname))
 	    {
-	      backend_system_device =
-		malloc (sizeof (struct backend_system_device));
-	      snprintf (backend_system_device->id, LABEL_MAX, "%s",
-			iportname);
-	      snprintf (backend_system_device->name, LABEL_MAX, "%s",
-			iportname);
-	      g_array_append_vals (devices, backend_system_device, 1);
+	      backend_device = g_malloc (sizeof (struct backend_device));
+	      backend_device->type = BE_TYPE_MIDI;
+	      snprintf (backend_device->id, LABEL_MAX, "%s", iportname);
+	      snprintf (backend_device->name, LABEL_MAX, "%s", iportname);
+	      g_array_append_vals (devices, backend_device, 1);
 	    }
 #else
 	  //We consider the cartesian product of inputs and outputs as the available ports.
-	  backend_system_device =
-	    malloc (sizeof (struct backend_system_device));
-	  snprintf (backend_system_device->id, LABEL_MAX, "%s%s%s",
-		    iportname, WINDOWS_INPUT_OUTPUT_SEPARATOR, oportname);
-	  snprintf (backend_system_device->name, LABEL_MAX, "%s%s%s",
-		    iportname, WINDOWS_INPUT_OUTPUT_SEPARATOR, oportname);
-	  g_array_append_vals (devices, backend_system_device, 1);
+	  backend_device = g_malloc (sizeof (struct backend_device));
+	  backend_device->type = BE_TYPE_MIDI;
+	  snprintf (backend_device->id, LABEL_MAX, "%s%s%s", iportname,
+		    WINDOWS_INPUT_OUTPUT_SEPARATOR, oportname);
+	  snprintf (backend_device->name, LABEL_MAX, "%s%s%s", iportname,
+		    WINDOWS_INPUT_OUTPUT_SEPARATOR, oportname);
+	  g_array_append_vals (devices, backend_device, 1);
 #endif
 	}
     }
@@ -293,9 +289,6 @@ cleanup_output:
 cleanup_input:
   rtmidi_close_port (outputp);
   rtmidi_out_free (outputp);
-
-end:
-  return devices;
 }
 
 const gchar *
