@@ -18,9 +18,9 @@
  *   along with Elektroid. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <glib/gi18n.h>
 #include "backend.h"
 #include "connector.h"
+#include "connectors/system.h"
 #include "connectors/elektron.h"
 #include "connectors/microbrute.h"
 #include "connectors/cz.h"
@@ -28,16 +28,7 @@
 #include "connectors/efactor.h"
 #include "connectors/phatty.h"
 #include "connectors/summit.h"
-
-static gint
-default_handshake (struct backend *backend)
-{
-  backend_midi_handshake (backend);
-  backend->filesystems = 0;
-  backend->fs_ops = NULL;
-  snprintf (backend->name, LABEL_MAX, "%s", _("MIDI device"));
-  return 0;
-}
+#include "connectors/default.h"
 
 struct connector
 {
@@ -108,7 +99,7 @@ static const struct connector *CONNECTORS[] = {
 
 gint
 connector_init_backend (struct backend *backend,
-			struct backend_system_device *device,
+			struct backend_device *device,
 			const gchar * conn_name,
 			struct sysex_transfer *sysex_transfer)
 {
@@ -117,6 +108,12 @@ connector_init_backend (struct backend *backend,
   gboolean active = TRUE;
   const struct connector **connector;
 
+  if (device->type == BE_TYPE_SYSTEM &&
+      !system_init_backend (backend, device->id))
+    {
+      return 0;
+    }
+
   err = backend_init (backend, device->id);
   if (err)
     {
