@@ -50,8 +50,11 @@ common_slot_get_id_name_from_path (const char *path, guint * id,
       goto end;
     }
 
-  if (strncmp (remainder, BE_SAMPLE_ID_NAME_SEPARATOR,
-	       strlen (BE_SAMPLE_ID_NAME_SEPARATOR)))
+  if (*remainder == G_SEARCHPATH_SEPARATOR)
+    {
+      remainder++;		//Skip ':'
+    }
+  else
     {
       if (name)
 	{
@@ -60,10 +63,6 @@ common_slot_get_id_name_from_path (const char *path, guint * id,
 	  goto end;
 	}
     }
-  else
-    {
-      remainder++;		//Skip ':'
-    }
 
   if (name)
     {
@@ -85,7 +84,7 @@ end:
 gchar *
 common_get_id_as_slot (struct item *item, struct backend *backend)
 {
-  gchar *slot = malloc (LABEL_MAX);
+  gchar *slot = g_malloc (LABEL_MAX);
   snprintf (slot, LABEL_MAX, "%d", item->id);
   return slot;
 }
@@ -95,16 +94,21 @@ common_print_item (struct item_iterator *iter, struct backend *backend,
 		   const struct fs_operations *fs_ops)
 {
   gchar *slot = NULL;
-  if (fs_ops->get_slot)
+  if (fs_ops->options & FS_OPTION_SLOT_STORAGE)
     {
-      slot = fs_ops->get_slot (&iter->item, backend);
+      if (fs_ops->get_slot)
+	{
+	  slot = fs_ops->get_slot (&iter->item, backend);
+	}
+      else
+	{
+	  slot = common_get_id_as_slot (&iter->item, backend);
+	}
     }
-  else
-    {
-      slot = common_get_id_as_slot (&iter->item, backend);
-    }
-  printf ("%c % 4" PRId64 "B %10s%s%s\n", iter->item.type, iter->item.size,
-	  slot ? slot : "", slot ? " " : "", iter->item.name);
+  gchar *hsize = get_human_size (iter->item.size, FALSE);
+  printf ("%c %10s %.*s%s%s\n", iter->item.type, hsize, slot ? 10 : 0,
+	  slot, slot ? " " : "", iter->item.name);
+  g_free (hsize);
   g_free (slot);
 }
 
@@ -119,7 +123,7 @@ common_midi_program_change (struct backend *backend, const gchar * dir,
   backend_program_change (backend, 0, item->id);
 }
 
-guint
+gint
 common_simple_next_dentry (struct item_iterator *iter)
 {
   struct common_simple_read_dir_data *data = iter->data;
@@ -134,12 +138,13 @@ common_simple_next_dentry (struct item_iterator *iter)
   iter->item.id = data->next;
   iter->item.type = ELEKTROID_FILE;
   iter->item.size = -1;
+  iter->item.slot_used = TRUE;
   data->next++;
 
   return 0;
 }
 
-guint
+gint
 common_data_upload (struct backend *backend, GByteArray * msg,
 		    struct job_control *control)
 {
@@ -178,7 +183,7 @@ cleanup:
 }
 
 
-guint
+gint
 common_data_download_part (struct backend *backend, GByteArray * tx_msg,
 			   GByteArray ** rx_msg, struct job_control *control)
 {
@@ -212,7 +217,7 @@ cleanup:
   return err;
 }
 
-guint
+gint
 common_data_download (struct backend *backend, GByteArray * tx_msg,
 		      GByteArray ** rx_msg, struct job_control *control)
 {
