@@ -144,7 +144,7 @@ efactor_get_download_path (struct backend *backend,
   return path;
 }
 
-static guint
+static gint
 efactor_next_dentry (struct item_iterator *iter)
 {
   struct efactor_iter_data *data = iter->data;
@@ -161,6 +161,7 @@ efactor_next_dentry (struct item_iterator *iter)
   snprintf (iter->item.name, LABEL_MAX, "%s", preset_name);
   iter->item.type = ELEKTROID_FILE;
   iter->item.size = -1;
+  iter->item.slot_used = TRUE;
   data->next++;
 
   return 0;
@@ -168,7 +169,7 @@ efactor_next_dentry (struct item_iterator *iter)
 
 static gint
 efactor_read_dir (struct backend *backend, struct item_iterator *iter,
-		  const gchar * path)
+		  const gchar * path, const gchar ** extensions)
 {
   GByteArray *tx_msg;
   GByteArray *rx_msg;
@@ -231,7 +232,7 @@ efactor_download (struct backend *backend, const gchar * src_path,
 
   if (!data->lines)
     {
-      err = efactor_read_dir (backend, &iter, "/");
+      err = efactor_read_dir (backend, &iter, "/", NULL);
       if (err)
 	{
 	  return err;
@@ -398,7 +399,7 @@ efactor_rename (struct backend *backend, const gchar * src, const gchar * dst)
 static gchar *
 efactor_get_slot (struct item *item, struct backend *backend)
 {
-  gchar *slot = malloc (LABEL_MAX);
+  gchar *slot = g_malloc (LABEL_MAX);
   struct efactor_data *data = backend->data;
   if (data->type == EFACTOR_FACTOR)
     {
