@@ -213,7 +213,7 @@ microbrute_get_download_path (struct backend *backend,
 
 static gint
 microbrute_read_dir (struct backend *backend, struct item_iterator *iter,
-		     const gchar * path)
+		     const gchar * path, const gchar ** extensions)
 {
   struct common_simple_read_dir_data *data;
 
@@ -516,7 +516,7 @@ microbrute_handshake (struct backend *backend)
       return -ENODEV;
     }
 
-  seq = malloc (sizeof (guint8));
+  seq = g_malloc (sizeof (guint8));
   *seq = 0;
 
   backend->filesystems = FS_MICROBRUTE_SEQUENCE;
