@@ -208,7 +208,7 @@ phatty_get_preset_dump_msg (guint8 id)
   return tx_msg;
 }
 
-static guint
+static gint
 phatty_next_root_dentry (struct item_iterator *iter)
 {
   guint *next = iter->data;
@@ -218,6 +218,7 @@ phatty_next_root_dentry (struct item_iterator *iter)
       iter->item.id = 0x1000;
       iter->item.type = ELEKTROID_DIR;
       iter->item.size = -1;
+      iter->item.slot_used = TRUE;
     }
   else if (*next == 1)
     {
@@ -225,6 +226,7 @@ phatty_next_root_dentry (struct item_iterator *iter)
       iter->item.id = PHATTY_PANEL_ID;
       iter->item.type = ELEKTROID_FILE;
       iter->item.size = -1;
+      iter->item.slot_used = TRUE;
     }
   else
     {
@@ -236,7 +238,7 @@ phatty_next_root_dentry (struct item_iterator *iter)
   return 0;
 }
 
-static guint
+static gint
 phatty_next_preset_dentry (struct item_iterator *iter)
 {
   gchar preset_name[MOOG_NAME_LEN + 1];
@@ -260,6 +262,7 @@ phatty_next_preset_dentry (struct item_iterator *iter)
   iter->item.id = data->next;
   iter->item.type = ELEKTROID_FILE;
   iter->item.size = PHATTY_PROGRAM_SIZE;
+  iter->item.slot_used = TRUE;
   (data->next)++;
 
   free_msg (rx_msg);
@@ -268,7 +271,7 @@ phatty_next_preset_dentry (struct item_iterator *iter)
 
 static gint
 phatty_read_dir (struct backend *backend, struct item_iterator *iter,
-		 const gchar * path)
+		 const gchar * path, const gchar ** extensions)
 {
   gint err = 0;
 
@@ -301,7 +304,7 @@ phatty_read_dir (struct backend *backend, struct item_iterator *iter,
 gchar *
 phatty_get_id_as_slot (struct item *item, struct backend *backend)
 {
-  gchar *slot = malloc (LABEL_MAX);
+  gchar *slot = g_malloc (LABEL_MAX);
   if (item->id >= PHATTY_MAX_PRESETS)
     {
       slot[0] = 0;
@@ -425,7 +428,8 @@ static const struct fs_operations FS_PHATTY_PRESET_OPERATIONS = {
   .fs = FS_PHATTY_PRESET,
   .options = FS_OPTION_SINGLE_OP | FS_OPTION_ID_AS_FILENAME |
     FS_OPTION_SLOT_STORAGE | FS_OPTION_SORT_BY_ID |
-    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SLOT_COLUMN,
+    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SLOT_COLUMN |
+    FS_OPTION_ALLOW_SEARCH,
   .name = "preset",
   .gui_name = "Presets",
   .gui_icon = BE_FILE_ICON_SND,
@@ -447,7 +451,7 @@ static const struct fs_operations FS_PHATTY_PRESET_OPERATIONS = {
 
 static gint
 phatty_scale_read_dir (struct backend *backend, struct item_iterator *iter,
-		       const gchar * path)
+		       const gchar * path, const gchar ** extensions)
 {
   struct common_simple_read_dir_data *data;
 
