@@ -24,6 +24,7 @@
 #include "elektron.h"
 #include "sample.h"
 #include "sds.h"
+#include "default.h"
 #include "common.h"
 
 #define SDS_SAMPLE_LIMIT 1000
@@ -201,25 +202,32 @@ sds_tx_handshake (struct backend *backend, const guint8 * msg, guint8 packet)
 
 static guint
 sds_get_download_info (GByteArray * header, struct sample_info *sample_info,
-		       guint * words, guint * word_size,
+		       guint * bits, guint * words, guint * word_size,
 		       guint * bytes_per_word)
 {
-  sample_info->bitdepth = header->data[6];
-  if (sds_get_bytes_per_word (sample_info->bitdepth, word_size,
-			      bytes_per_word))
+  *bits = header->data[6];
+  if (*bits == 8)
+    {
+      sample_info->format = SF_FORMAT_WAV | SF_FORMAT_PCM_U8;
+    }
+  else
+    {
+      sample_info->format = SF_FORMAT_WAV | SF_FORMAT_PCM_16;
+    }
+  if (sds_get_bytes_per_word (*bits, word_size, bytes_per_word))
     {
       return -1;
     }
-  sample_info->samplerate =
-    1.0e9 / sds_get_bytes_value_right_just (&header->data[7],
-					    SDS_BYTES_PER_WORD);
+  sample_info->rate = 1.0e9 /
+    sds_get_bytes_value_right_just (&header->data[7], SDS_BYTES_PER_WORD);
   *words =
     sds_get_bytes_value_right_just (&header->data[10], SDS_BYTES_PER_WORD);
-  sample_info->loopstart =
+  sample_info->loop_start =
     sds_get_bytes_value_right_just (&header->data[13], SDS_BYTES_PER_WORD);
-  sample_info->loopend =
+  sample_info->loop_end =
     sds_get_bytes_value_right_just (&header->data[16], SDS_BYTES_PER_WORD);
-  sample_info->looptype = header->data[19];
+  sample_info->loop_type = header->data[19];
+  sample_info->midi_note = 0;
   sample_info->channels = 1;
   return 0;
 }
@@ -259,18 +267,18 @@ sds_get_dump_msg (guint id, guint frames, struct sample_info *sample_info,
   if (sample_info)
     {
       tx_msg->data[6] = (guint8) bits;
-      period = 1.0e9 / sample_info->samplerate;
+      period = 1.0e9 / sample_info->rate;
       sds_set_bytes_value_right_just (&tx_msg->data[7], SDS_BYTES_PER_WORD,
 				      period);
       sds_set_bytes_value_right_just (&tx_msg->data[10], SDS_BYTES_PER_WORD,
 				      frames);
       sds_set_bytes_value_right_just (&tx_msg->data[13], SDS_BYTES_PER_WORD,
-				      sample_info->loopstart);
+				      sample_info->loop_start);
       sds_set_bytes_value_right_just (&tx_msg->data[16], SDS_BYTES_PER_WORD,
-				      sample_info->loopend);
-      tx_msg->data[19] = (sample_info->loopstart == sample_info->loopend
-			  && sample_info->loopstart ==
-			  frames - 1) ? 0x7f : sample_info->looptype;
+				      sample_info->loop_end);
+      tx_msg->data[19] = (sample_info->loop_start == sample_info->loop_end
+			  && sample_info->loop_start ==
+			  frames - 1) ? 0x7f : sample_info->loop_type;
     }
 
   return tx_msg;
@@ -302,13 +310,13 @@ sds_download_inc_packet (gboolean * first, guint * packet)
 }
 
 static void
-sds_debug_print_sample_data (guint bitdepth, guint bytes_per_word,
+sds_debug_print_sample_data (guint bits, guint bytes_per_word,
 			     guint word_size, guint sample_rate, guint words,
 			     guint packets)
 {
   debug_print (1,
 	       "Resolution: %d bits; %d bytes per word; word size %d bytes.\n",
-	       bitdepth, bytes_per_word, word_size);
+	       bits, bytes_per_word, word_size);
   debug_print (1, "Sample rate: %d Hz\n", sample_rate);
   debug_print (1, "Words: %d\n", words);
   debug_print (1, "Packets: %d\n", packets);
@@ -341,7 +349,7 @@ sds_download_try (struct backend *backend, const gchar * path,
 		  GByteArray * output, struct job_control *control)
 {
   guint id, words, word_size, read_bytes, bytes_per_word, total_words, err,
-    retries, packets, packet, exp_packet, rx_packets;
+    retries, packets, packet, exp_packet, rx_packets, bits;
   gint16 sample;
   GByteArray *tx_msg, *rx_msg;
   gchar *name;
@@ -389,8 +397,8 @@ sds_download_try (struct backend *backend, const gchar * path,
 	}
     }
 
-  sample_info = malloc (sizeof (struct sample_info));
-  if (sds_get_download_info (rx_msg, sample_info, &words, &word_size,
+  sample_info = g_malloc (sizeof (struct sample_info));
+  if (sds_get_download_info (rx_msg, sample_info, &bits, &words, &word_size,
 			     &bytes_per_word))
     {
       free_msg (rx_msg);
@@ -401,9 +409,8 @@ sds_download_try (struct backend *backend, const gchar * path,
 
   packets =
     ceil (words / (double) (SDS_DATA_PACKET_PAYLOAD_LEN / bytes_per_word));
-  sds_debug_print_sample_data (sample_info->bitdepth, bytes_per_word,
-			       word_size, sample_info->samplerate, words,
-			       packets);
+  sds_debug_print_sample_data (bits, bytes_per_word, word_size,
+			       sample_info->rate, words, packets);
 
   g_mutex_lock (&control->mutex);
   active = control->active;
@@ -522,9 +529,8 @@ sds_download_try (struct backend *backend, const gchar * path,
       dataptr = &rx_msg->data[5];
       while (read_bytes < SDS_DATA_PACKET_PAYLOAD_LEN && total_words < words)
 	{
-	  sample = sds_get_gint16_value_left_just (dataptr,
-						   bytes_per_word,
-						   sample_info->bitdepth);
+	  sample = sds_get_gint16_value_left_just (dataptr, bytes_per_word,
+						   bits);
 	  g_byte_array_append (output, (guint8 *) & sample, sizeof (sample));
 	  dataptr += bytes_per_word;
 	  read_bytes += bytes_per_word;
@@ -798,8 +804,7 @@ sds_upload (struct backend *backend, const gchar * path, GByteArray * input,
 
   word = 0;
   sds_debug_print_sample_data (bits, bytes_per_word,
-			       word_size, sample_info->samplerate, words,
-			       packets);
+			       word_size, sample_info->rate, words, packets);
   frame = (gint16 *) input->data;
   while (packet < packets && active)
     {
@@ -933,7 +938,7 @@ sds_upload_16b (struct backend *backend, const gchar * path,
 
 static gint
 sds_read_dir (struct backend *backend, struct item_iterator *iter,
-	      const gchar * path)
+	      const gchar * path, const gchar ** extensions)
 {
   struct common_simple_read_dir_data *data;
 
@@ -952,32 +957,84 @@ sds_read_dir (struct backend *backend, struct item_iterator *iter,
   return 0;
 }
 
-gint
+static gint
+sds_sample_load_with_rate (const gchar * path, GByteArray * sample,
+			   struct job_control *control, guint32 rate)
+{
+  struct sample_info sample_info_dst;
+  sample_info_dst.rate = rate;
+  sample_info_dst.channels = SDS_SAMPLE_CHANNELS;
+  sample_info_dst.format = SF_FORMAT_PCM_16;
+  gint res = sample_load_from_file (path, sample, control, &sample_info_dst);
+  if (!res)
+    {
+      memcpy (control->data, &sample_info_dst, sizeof (struct sample_info));
+    }
+  return res;
+}
+
+static gint
 sds_sample_load (const gchar * path, GByteArray * sample,
 		 struct job_control *control)
 {
-  guint frames;
-  struct sample_params sample_params;
-  sample_params.samplerate = 0;	// Any sample rate is valid.
-  sample_params.channels = SDS_SAMPLE_CHANNELS;
-  return sample_load_from_file (path, sample, control, &sample_params,
-				&frames);
+  return sds_sample_load_with_rate (path, sample, control, 0);	// Any sample rate is valid.
+}
+
+static gint
+sds_sample_load_441 (const gchar * path, GByteArray * sample,
+		     struct job_control *control)
+{
+  return sds_sample_load_with_rate (path, sample, control, 44100);
+}
+
+static gint
+sds_sample_load_32 (const gchar * path, GByteArray * sample,
+		    struct job_control *control)
+{
+  return sds_sample_load_with_rate (path, sample, control, 32000);
+}
+
+static gint
+sds_sample_load_16 (const gchar * path, GByteArray * sample,
+		    struct job_control *control)
+{
+  return sds_sample_load_with_rate (path, sample, control, 16000);
+}
+
+static gint
+sds_sample_load_8 (const gchar * path, GByteArray * sample,
+		   struct job_control *control)
+{
+  return sds_sample_load_with_rate (path, sample, control, 8000);
+}
+
+static gint
+sds_sample_save (const gchar * path, GByteArray * sample,
+		 struct job_control *control)
+{
+  return sample_save_to_file (path, sample, control,
+			      SF_FORMAT_WAV | SF_FORMAT_PCM_16);
 }
 
 enum sds_fs
 {
-  FS_SAMPLES_SDS_16_B = 0x1,
-  FS_SAMPLES_SDS_14_B = 0x2,
-  FS_SAMPLES_SDS_12_B = 0x4,
-  FS_SAMPLES_SDS_8_B = 0x8
+  FS_SAMPLES_DEFAULT = FS_PROGRAM_DEFAULT,
+  FS_SAMPLES_SDS_16_B = 0x2,
+  FS_SAMPLES_SDS_14_B = 0x4,
+  FS_SAMPLES_SDS_12_B = 0x8,
+  FS_SAMPLES_SDS_8_B = 0x10,
+  FS_SAMPLES_SDS_16_B_441 = 0x20,
+  FS_SAMPLES_SDS_16_B_32 = 0x40,
+  FS_SAMPLES_SDS_16_B_16 = 0x80,
+  FS_SAMPLES_SDS_16_B_8 = 0x100
 };
 
 static const struct fs_operations FS_SAMPLES_SDS_8B_OPERATIONS = {
   .fs = FS_SAMPLES_SDS_8_B,
-  .options = FS_OPTION_AUDIO_PLAYER | FS_OPTION_SINGLE_OP |
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO | FS_OPTION_SINGLE_OP |
     FS_OPTION_ID_AS_FILENAME | FS_OPTION_SLOT_STORAGE | FS_OPTION_SORT_BY_ID,
-  .name = "mono8",
-  .gui_name = "Mono 8 bits",
+  .name = "8bits1c",
+  .gui_name = "8 bits mono",
   .gui_icon = BE_FILE_ICON_WAVE,
   .type_ext = "wav",
   .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
@@ -987,7 +1044,7 @@ static const struct fs_operations FS_SAMPLES_SDS_8B_OPERATIONS = {
   .download = sds_download,
   .upload = sds_upload_8b,
   .load = sds_sample_load,
-  .save = sample_save_from_array,
+  .save = sds_sample_save,
   .get_ext = backend_get_fs_ext,
   .get_upload_path = common_slot_get_upload_path,
   .get_download_path = sds_get_download_path
@@ -995,10 +1052,10 @@ static const struct fs_operations FS_SAMPLES_SDS_8B_OPERATIONS = {
 
 static const struct fs_operations FS_SAMPLES_SDS_12B_OPERATIONS = {
   .fs = FS_SAMPLES_SDS_12_B,
-  .options = FS_OPTION_AUDIO_PLAYER | FS_OPTION_SINGLE_OP |
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO | FS_OPTION_SINGLE_OP |
     FS_OPTION_ID_AS_FILENAME | FS_OPTION_SLOT_STORAGE | FS_OPTION_SORT_BY_ID,
-  .name = "mono12",
-  .gui_name = "Mono 12 bits",
+  .name = "12bits1c",
+  .gui_name = "12 bits mono",
   .gui_icon = BE_FILE_ICON_WAVE,
   .type_ext = "wav",
   .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
@@ -1008,7 +1065,7 @@ static const struct fs_operations FS_SAMPLES_SDS_12B_OPERATIONS = {
   .download = sds_download,
   .upload = sds_upload_12b,
   .load = sds_sample_load,
-  .save = sample_save_from_array,
+  .save = sds_sample_save,
   .get_ext = backend_get_fs_ext,
   .get_upload_path = common_slot_get_upload_path,
   .get_download_path = sds_get_download_path
@@ -1016,10 +1073,10 @@ static const struct fs_operations FS_SAMPLES_SDS_12B_OPERATIONS = {
 
 static const struct fs_operations FS_SAMPLES_SDS_14B_OPERATIONS = {
   .fs = FS_SAMPLES_SDS_14_B,
-  .options = FS_OPTION_AUDIO_PLAYER | FS_OPTION_SINGLE_OP |
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO | FS_OPTION_SINGLE_OP |
     FS_OPTION_ID_AS_FILENAME | FS_OPTION_SLOT_STORAGE | FS_OPTION_SORT_BY_ID,
-  .name = "mono14",
-  .gui_name = "Mono 14 bits",
+  .name = "14bits1c",
+  .gui_name = "14 bits mono",
   .gui_icon = BE_FILE_ICON_WAVE,
   .type_ext = "wav",
   .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
@@ -1029,7 +1086,7 @@ static const struct fs_operations FS_SAMPLES_SDS_14B_OPERATIONS = {
   .download = sds_download,
   .upload = sds_upload_14b,
   .load = sds_sample_load,
-  .save = sample_save_from_array,
+  .save = sds_sample_save,
   .get_ext = backend_get_fs_ext,
   .get_upload_path = common_slot_get_upload_path,
   .get_download_path = sds_get_download_path
@@ -1037,10 +1094,10 @@ static const struct fs_operations FS_SAMPLES_SDS_14B_OPERATIONS = {
 
 static const struct fs_operations FS_SAMPLES_SDS_16B_OPERATIONS = {
   .fs = FS_SAMPLES_SDS_16_B,
-  .options = FS_OPTION_AUDIO_PLAYER | FS_OPTION_SINGLE_OP |
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO | FS_OPTION_SINGLE_OP |
     FS_OPTION_ID_AS_FILENAME | FS_OPTION_SLOT_STORAGE | FS_OPTION_SORT_BY_ID,
-  .name = "mono16",
-  .gui_name = "Mono 16 bits",
+  .name = "16bits1c",
+  .gui_name = "16 bits mono",
   .gui_icon = BE_FILE_ICON_WAVE,
   .type_ext = "wav",
   .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
@@ -1050,15 +1107,102 @@ static const struct fs_operations FS_SAMPLES_SDS_16B_OPERATIONS = {
   .download = sds_download,
   .upload = sds_upload_16b,
   .load = sds_sample_load,
-  .save = sample_save_from_array,
+  .save = sds_sample_save,
+  .get_ext = backend_get_fs_ext,
+  .get_upload_path = common_slot_get_upload_path,
+  .get_download_path = sds_get_download_path
+};
+
+static const struct fs_operations FS_SAMPLES_SDS_16B_441_OPERATIONS = {
+  .fs = FS_SAMPLES_SDS_16_B_441,
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO | FS_OPTION_SINGLE_OP |
+    FS_OPTION_ID_AS_FILENAME | FS_OPTION_SLOT_STORAGE | FS_OPTION_SORT_BY_ID,
+  .name = "44.1k16b1c",
+  .gui_name = "44.1 KHz 16 bits mono",
+  .gui_icon = BE_FILE_ICON_WAVE,
+  .type_ext = "wav",
+  .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
+  .readdir = sds_read_dir,
+  .print_item = common_print_item,
+  .rename = sds_rename,
+  .download = sds_download,
+  .upload = sds_upload_16b,
+  .load = sds_sample_load_441,
+  .save = sds_sample_save,
+  .get_ext = backend_get_fs_ext,
+  .get_upload_path = common_slot_get_upload_path,
+  .get_download_path = sds_get_download_path
+};
+
+static const struct fs_operations FS_SAMPLES_SDS_16B_32_OPERATIONS = {
+  .fs = FS_SAMPLES_SDS_16_B_32,
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO | FS_OPTION_SINGLE_OP |
+    FS_OPTION_ID_AS_FILENAME | FS_OPTION_SLOT_STORAGE | FS_OPTION_SORT_BY_ID,
+  .name = "32k16b1c",
+  .gui_name = "32 KHz 16 bits mono",
+  .gui_icon = BE_FILE_ICON_WAVE,
+  .type_ext = "wav",
+  .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
+  .readdir = sds_read_dir,
+  .print_item = common_print_item,
+  .rename = sds_rename,
+  .download = sds_download,
+  .upload = sds_upload_16b,
+  .load = sds_sample_load_32,
+  .save = sds_sample_save,
+  .get_ext = backend_get_fs_ext,
+  .get_upload_path = common_slot_get_upload_path,
+  .get_download_path = sds_get_download_path
+};
+
+static const struct fs_operations FS_SAMPLES_SDS_16B_16_OPERATIONS = {
+  .fs = FS_SAMPLES_SDS_16_B_16,
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO | FS_OPTION_SINGLE_OP |
+    FS_OPTION_ID_AS_FILENAME | FS_OPTION_SLOT_STORAGE | FS_OPTION_SORT_BY_ID,
+  .name = "16k16b1c",
+  .gui_name = "16 KHz 16 bits mono",
+  .gui_icon = BE_FILE_ICON_WAVE,
+  .type_ext = "wav",
+  .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
+  .readdir = sds_read_dir,
+  .print_item = common_print_item,
+  .rename = sds_rename,
+  .download = sds_download,
+  .upload = sds_upload_16b,
+  .load = sds_sample_load_16,
+  .save = sds_sample_save,
+  .get_ext = backend_get_fs_ext,
+  .get_upload_path = common_slot_get_upload_path,
+  .get_download_path = sds_get_download_path
+};
+
+static const struct fs_operations FS_SAMPLES_SDS_16B_8_OPERATIONS = {
+  .fs = FS_SAMPLES_SDS_16_B_8,
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO | FS_OPTION_SINGLE_OP |
+    FS_OPTION_ID_AS_FILENAME | FS_OPTION_SLOT_STORAGE | FS_OPTION_SORT_BY_ID,
+  .name = "8k16b1c",
+  .gui_name = "8 KHz 16 bits mono",
+  .gui_icon = BE_FILE_ICON_WAVE,
+  .type_ext = "wav",
+  .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
+  .readdir = sds_read_dir,
+  .print_item = common_print_item,
+  .rename = sds_rename,
+  .download = sds_download,
+  .upload = sds_upload_16b,
+  .load = sds_sample_load_8,
+  .save = sds_sample_save,
   .get_ext = backend_get_fs_ext,
   .get_upload_path = common_slot_get_upload_path,
   .get_download_path = sds_get_download_path
 };
 
 static const struct fs_operations *FS_SDS_ALL_OPERATIONS[] = {
-  &FS_SAMPLES_SDS_8B_OPERATIONS, &FS_SAMPLES_SDS_12B_OPERATIONS,
-  &FS_SAMPLES_SDS_14B_OPERATIONS, &FS_SAMPLES_SDS_16B_OPERATIONS, NULL
+  &FS_PROGRAM_DEFAULT_OPERATIONS, &FS_SAMPLES_SDS_8B_OPERATIONS,
+  &FS_SAMPLES_SDS_12B_OPERATIONS, &FS_SAMPLES_SDS_14B_OPERATIONS,
+  &FS_SAMPLES_SDS_16B_OPERATIONS, &FS_SAMPLES_SDS_16B_441_OPERATIONS,
+  &FS_SAMPLES_SDS_16B_32_OPERATIONS, &FS_SAMPLES_SDS_16B_16_OPERATIONS,
+  &FS_SAMPLES_SDS_16B_8_OPERATIONS, NULL
 };
 
 gint
@@ -1125,8 +1269,10 @@ sds_handshake (struct backend *backend)
 
   sds_data->rest_time = SDS_REST_TIME_DEFAULT;
 
-  backend->filesystems = FS_SAMPLES_SDS_8_B | FS_SAMPLES_SDS_12_B |
-    FS_SAMPLES_SDS_14_B | FS_SAMPLES_SDS_16_B;
+  backend->filesystems = FS_PROGRAM_DEFAULT | FS_SAMPLES_SDS_8_B |
+    FS_SAMPLES_SDS_12_B | FS_SAMPLES_SDS_14_B | FS_SAMPLES_SDS_16_B |
+    FS_SAMPLES_SDS_16_B_441 | FS_SAMPLES_SDS_16_B_32 | FS_SAMPLES_SDS_16_B_16
+    | FS_SAMPLES_SDS_16_B_8;
   backend->fs_ops = FS_SDS_ALL_OPERATIONS;
   backend->destroy_data = backend_destroy_data;
   backend->data = sds_data;
