@@ -144,7 +144,7 @@ summit_get_patch_download_path (struct backend *backend,
   return path;
 }
 
-static guint
+static gint
 summit_patch_next_dentry (struct item_iterator *iter)
 {
   GByteArray *tx_msg, *rx_msg;
@@ -173,6 +173,7 @@ summit_patch_next_dentry (struct item_iterator *iter)
   iter->item.type = ELEKTROID_FILE;
   iter->item.size =
     data->fs == FS_SUMMIT_SINGLE_PATCH ? SUMMIT_SINGLE_LEN : SUMMIT_MULTI_LEN;
+  iter->item.slot_used = TRUE;
   data->next++;
 
   usleep (SUMMIT_REST_TIME_US);
@@ -180,7 +181,7 @@ summit_patch_next_dentry (struct item_iterator *iter)
   return 0;
 }
 
-static guint
+static gint
 summit_patch_next_dentry_root (struct item_iterator *iter)
 {
   guint *next = iter->data;
@@ -191,6 +192,7 @@ summit_patch_next_dentry_root (struct item_iterator *iter)
       snprintf (iter->item.name, LABEL_MAX, "%c", 0x41 + iter->item.id);
       iter->item.type = ELEKTROID_DIR;
       iter->item.size = -1;
+      iter->item.slot_used = TRUE;
       (*next)++;
       return 0;
     }
@@ -234,14 +236,14 @@ summit_patch_read_dir (struct backend *backend, struct item_iterator *iter,
 
 static gint
 summit_single_read_dir (struct backend *backend, struct item_iterator *iter,
-			const gchar * path)
+			const gchar * path, const gchar ** extensions)
 {
   return summit_patch_read_dir (backend, iter, path, FS_SUMMIT_SINGLE_PATCH);
 }
 
 static gint
 summit_multi_read_dir (struct backend *backend, struct item_iterator *iter,
-		       const gchar * path)
+		       const gchar * path, const gchar ** extensions)
 {
   return summit_patch_read_dir (backend, iter, path, FS_SUMMIT_MULTI_PATCH);
 }
@@ -435,7 +437,7 @@ static gchar *
 summit_get_id_as_slot (struct item *item, struct backend *backend,
 		       gint digits)
 {
-  gchar *slot = malloc (LABEL_MAX);
+  gchar *slot = g_malloc (LABEL_MAX);
   if (item->id < BE_MAX_MIDI_PROGRAMS)
     {
       snprintf (slot, LABEL_MAX, "%.*d", digits, item->id);
@@ -497,7 +499,8 @@ static const struct fs_operations FS_SUMMIT_SINGLE_OPERATIONS = {
   .fs = FS_SUMMIT_SINGLE_PATCH,
   .options = FS_OPTION_SINGLE_OP | FS_OPTION_ID_AS_FILENAME |
     FS_OPTION_SLOT_STORAGE | FS_OPTION_SORT_BY_ID |
-    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SLOT_COLUMN,
+    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SLOT_COLUMN |
+    FS_OPTION_ALLOW_SEARCH,
   .name = "single",
   .gui_name = "Single",
   .gui_icon = BE_FILE_ICON_SND,
@@ -521,7 +524,8 @@ static const struct fs_operations FS_SUMMIT_MULTI_OPERATIONS = {
   .fs = FS_SUMMIT_MULTI_PATCH,
   .options = FS_OPTION_SINGLE_OP | FS_OPTION_ID_AS_FILENAME |
     FS_OPTION_SLOT_STORAGE | FS_OPTION_SORT_BY_ID |
-    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SLOT_COLUMN,
+    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SLOT_COLUMN |
+    FS_OPTION_ALLOW_SEARCH,
   .name = "multi",
   .gui_name = "Multi",
   .gui_icon = BE_FILE_ICON_SND,
@@ -543,7 +547,7 @@ static const struct fs_operations FS_SUMMIT_MULTI_OPERATIONS = {
 
 static gint
 summit_scale_read_dir (struct backend *backend, struct item_iterator *iter,
-		       const gchar * path)
+		       const gchar * path, const gchar ** extensions)
 {
   struct common_simple_read_dir_data *data;
 
@@ -686,7 +690,7 @@ summit_get_wavetable_wave_dump_msg (guint id, guint8 wave)
   return tx_msg;
 }
 
-static guint
+static gint
 summit_wavetable_next_dentry (struct item_iterator *iter)
 {
   GByteArray *tx_msg, *rx_msg;
@@ -713,6 +717,7 @@ summit_wavetable_next_dentry (struct item_iterator *iter)
   iter->item.id = data->next;
   iter->item.type = ELEKTROID_FILE;
   iter->item.size = 2678;
+  iter->item.slot_used = TRUE;
   data->next++;
 
   usleep (SUMMIT_REST_TIME_US * 10);
@@ -728,7 +733,8 @@ summit_get_wavetable_id_as_slot (struct item *item, struct backend *backend)
 
 static gint
 summit_wavetable_read_dir (struct backend *backend,
-			   struct item_iterator *iter, const gchar * path)
+			   struct item_iterator *iter, const gchar * path,
+			   const gchar ** extensions)
 {
   if (!strcmp (path, "/"))
     {
