@@ -31,6 +31,8 @@
 #include "connector.h"
 #include "utils.h"
 
+#define COMMAND_NOT_IN_SYSTEM_FS "Command not available in system backend\n"
+
 #define GET_FS_OPS_OFFSET(member) offsetof(struct fs_operations, member)
 #define GET_FS_OPS_FUNC(type,fs,offset) (*(((type *) (((gchar *) fs) + offset))))
 #define CHECK_FS_OPS_FUNC(f) if (!(f)) {return -ENOSYS;}
@@ -48,11 +50,12 @@ cli_get_path (gchar * device_path)
   gchar *path = device_path;
   gint i = 0;
 
-  while (*path != '/' && i < len)
+  while (*path != G_SEARCHPATH_SEPARATOR && i < len)
     {
       path++;
       i++;
     }
+  path++;
 
   return path;
 }
@@ -61,12 +64,12 @@ static gint
 cli_ld ()
 {
   gint i;
-  struct backend_system_device device;
-  GArray *devices = backend_get_system_devices ();
+  struct backend_device device;
+  GArray *devices = backend_get_devices ();
 
   for (i = 0; i < devices->len; i++)
     {
-      device = g_array_index (devices, struct backend_system_device, i);
+      device = g_array_index (devices, struct backend_device, i);
       printf ("%d: id: %s; name: %s\n", i, device.id, device.name);
     }
 
@@ -79,8 +82,8 @@ static gint
 cli_connect (const gchar * device_path)
 {
   gint err, id = (gint) atoi (device_path);
-  struct backend_system_device device;
-  GArray *devices = backend_get_system_devices ();
+  struct backend_device device;
+  GArray *devices = backend_get_devices ();
 
   if (!devices->len || id >= devices->len)
     {
@@ -88,7 +91,7 @@ cli_connect (const gchar * device_path)
       return -ENODEV;
     }
 
-  device = g_array_index (devices, struct backend_system_device, id);
+  device = g_array_index (devices, struct backend_device, id);
   err = connector_init_backend (&backend, &device, connector, NULL);
   g_array_free (devices, TRUE);
 
@@ -134,7 +137,7 @@ cli_list (int argc, gchar * argv[], int *optind)
 
   CHECK_FS_OPS_FUNC (fs_ops->readdir);
 
-  err = fs_ops->readdir (&backend, &iter, path);
+  err = fs_ops->readdir (&backend, &iter, path, NULL);
   if (err)
     {
       return err;
@@ -323,11 +326,14 @@ cli_info (int argc, gchar * argv[], int *optind)
       return err;
     }
 
-  printf ("%s; version: %s; description: %s; connector: %s; filesystems: ",
-	  backend.name, backend.version, backend.description,
-	  backend.conn_name);
+  printf ("Type: %s\n", backend.type == BE_TYPE_SYSTEM ? "SYSTEM" : "MIDI");
+  printf ("Device name: %s\n", backend.name);
+  printf ("Device version: %s\n", backend.version);
+  printf ("Device description: %s\n", backend.description);
+  printf ("Connector name: %s\n", backend.conn_name);
+  printf ("Filesystems: ");
 
-  for (gint fs = 1; fs <= MAX_BACKEND_FSS; fs <<= 1)
+  for (guint32 fs = 1, i = 0; i < MAX_BACKEND_FSS; fs <<= 1, i++)
     {
       if (backend.filesystems & fs)
 	{
@@ -344,6 +350,7 @@ static int
 cli_df (int argc, gchar * argv[], int *optind)
 {
   gchar *device_path;
+  const gchar *path;
   gchar *size;
   gchar *diff;
   gchar *free;
@@ -372,7 +379,13 @@ cli_df (int argc, gchar * argv[], int *optind)
       return -ENOSYS;
     }
 
-  printf ("%-10.10s%16.16s%16.16s%16.16s%11.10s\n", "Storage", "Size",
+  path = cli_get_path (device_path);
+  if (!strlen (path))
+    {
+      return -EINVAL;
+    }
+
+  printf ("%-20.20s%16.16s%16.16s%16.16s%11.10s\n", "Storage", "Size",
 	  "Used", "Available", "Use%");
 
   err = 0;
@@ -380,7 +393,7 @@ cli_df (int argc, gchar * argv[], int *optind)
     {
       if (backend.storage & storage)
 	{
-	  err |= backend.get_storage_stats (&backend, storage, &statfs);
+	  err |= backend.get_storage_stats (&backend, storage, &statfs, path);
 	  if (err)
 	    {
 	      continue;
@@ -388,7 +401,7 @@ cli_df (int argc, gchar * argv[], int *optind)
 	  size = get_human_size (statfs.bsize, FALSE);
 	  diff = get_human_size (statfs.bsize - statfs.bfree, FALSE);
 	  free = get_human_size (statfs.bfree, FALSE);
-	  printf ("%-10.10s%16s%16s%16s%10.2f%%\n",
+	  printf ("%-20.20s%16s%16s%16s%10.2f%%\n",
 		  statfs.name, size, diff, free,
 		  backend_get_storage_stats_percent (&statfs));
 	  g_free (size);
@@ -396,7 +409,7 @@ cli_df (int argc, gchar * argv[], int *optind)
 	  g_free (free);
 	}
 
-      sleep (BE_REST_TIME_US);
+      usleep (BE_REST_TIME_US);
     }
 
   return err;
@@ -436,6 +449,11 @@ cli_upgrade_os (int argc, gchar * argv[], int *optind)
     {
       return err;
     }
+  if (backend.type == BE_TYPE_SYSTEM)
+    {
+      error_print (COMMAND_NOT_IN_SYSTEM_FS);
+      return EXIT_FAILURE;
+    }
 
   sysex_transfer.raw = g_byte_array_new ();
   err = load_file (src_path, sysex_transfer.raw, NULL);
@@ -596,12 +614,16 @@ cli_send (int argc, gchar * argv[], int *optind)
     }
 
   connector = "default";
-
   err = cli_connect (device_dst_path);
   if (err)
     {
       return err;
     }
+  if (backend.type == BE_TYPE_SYSTEM)
+    {
+      error_print (COMMAND_NOT_IN_SYSTEM_FS);
+      return EXIT_FAILURE;
+    }
 
   sysex_transfer.active = TRUE;
   sysex_transfer.timeout = BE_DUMP_TIMEOUT;
@@ -647,12 +669,16 @@ cli_receive (int argc, gchar * argv[], int *optind)
     }
 
   connector = "default";
-
   err = cli_connect (device_src_path);
   if (err)
     {
       return err;
     }
+  if (backend.type == BE_TYPE_SYSTEM)
+    {
+      error_print (COMMAND_NOT_IN_SYSTEM_FS);
+      return EXIT_FAILURE;
+    }
 
   sysex_transfer.timeout = BE_DUMP_TIMEOUT;
   sysex_transfer.batch = TRUE;
