@@ -27,7 +27,11 @@ struct preferences
 {
   gboolean autoplay;
   gboolean mix;
+  gboolean show_remote;
   gchar *local_dir;
+  gchar *remote_dir;
+  gboolean show_grid;
+  gint grid_length;
 };
 
 gint preferences_save (struct preferences *);
