@@ -18,22 +18,22 @@
  *   along with Elektroid. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <sndfile.h>
 #include <samplerate.h>
 #include <math.h>
 #include <errno.h>
 #include "sample.h"
 
-#define LOAD_BUFFER_LEN (8 * 1024)
+#define LOAD_BUFFER_LEN (32 * 1024)
 
 #define JUNK_CHUNK_ID "JUNK"
 #define SMPL_CHUNK_ID "smpl"
 
 static const gchar *ELEKTROID_AUDIO_LOCAL_EXTS[] =
-  { "wav", "ogg", "aiff", "flac", NULL };
-
-static const gchar *ELEKTROID_AUDIO_LOCAL_EXTS_MP3[] =
-  { "wav", "ogg", "aiff", "flac", "mp3", NULL };
+#if !defined(__linux__) || HAVE_SNDFILE_MP3
+{ "wav", "ogg", "aiff", "flac", "mp3", NULL };
+#else
+{ "wav", "ogg", "aiff", "flac", NULL };
+#endif
 
 struct smpl_chunk_data
 {
@@ -201,8 +201,8 @@ static SF_VIRTUAL_IO FILE_IO = {
 };
 
 static gint
-sample_get_wave_data (GByteArray * sample, struct job_control *control,
-		      struct g_byte_array_io_data *wave)
+sample_get_audio_file_data (GByteArray * sample, struct job_control *control,
+			    struct g_byte_array_io_data *wave, guint32 format)
 {
   SF_INFO sf_info;
   SNDFILE *sndfile;
@@ -212,19 +212,20 @@ sample_get_wave_data (GByteArray * sample, struct job_control *control,
   struct smpl_chunk_data smpl_chunk_data;
   struct sample_info *sample_info = control->data;
 
-  g_byte_array_set_size (wave->array, sample->len + 1024);	//We need space for the headers.
+  g_byte_array_set_size (wave->array, sample->len + 4096);	//We need space for the headers.
   wave->array->len = 0;
 
-  frames = sample->len >> sample_info->channels;
+  frames = sample->len / SAMPLE_INFO_FRAME_SIZE (sample_info);
   debug_print (1, "Frames: %" PRIu64 "; sample rate: %d; channels: %d\n",
-	       frames, sample_info->samplerate, sample_info->channels);
+	       frames, sample_info->rate, sample_info->channels);
   debug_print (1, "Loop start at %d; loop end at %d\n",
-	       sample_info->loopstart, sample_info->loopend);
+	       sample_info->loop_start, sample_info->loop_end);
 
   memset (&sf_info, 0, sizeof (sf_info));
-  sf_info.samplerate = sample_info->samplerate;
+  sf_info.samplerate = sample_info->rate;
   sf_info.channels = sample_info->channels;
-  sf_info.format = SF_FORMAT_WAV | SF_FORMAT_PCM_16;
+  sf_info.format = format;
+
   sndfile = sf_open_virtual (&G_BYTE_ARRAY_IO, SFM_WRITE, &sf_info, wave);
   if (!sndfile)
     {
@@ -243,17 +244,17 @@ sample_get_wave_data (GByteArray * sample, struct job_control *control,
 
   smpl_chunk_data.manufacturer = 0;
   smpl_chunk_data.product = 0;
-  smpl_chunk_data.sample_period = 1e9 / sample_info->samplerate;
-  smpl_chunk_data.midi_unity_note = 60;
+  smpl_chunk_data.sample_period = 1e9 / sample_info->rate;
+  smpl_chunk_data.midi_unity_note = sample_info->midi_note;
   smpl_chunk_data.midi_pitch_fraction = 0;
   smpl_chunk_data.smpte_format = 0;
   smpl_chunk_data.smpte_offset = 0;
   smpl_chunk_data.num_sampler_loops = 1;
   smpl_chunk_data.sampler_data = 0;
   smpl_chunk_data.sample_loop.cue_point_id = 0;
-  smpl_chunk_data.sample_loop.type = sample_info->looptype;
-  smpl_chunk_data.sample_loop.start = sample_info->loopstart;
-  smpl_chunk_data.sample_loop.end = sample_info->loopend;
+  smpl_chunk_data.sample_loop.type = sample_info->loop_type;
+  smpl_chunk_data.sample_loop.start = sample_info->loop_start;
+  smpl_chunk_data.sample_loop.end = sample_info->loop_end;
   smpl_chunk_data.sample_loop.fraction = 0;
   smpl_chunk_data.sample_loop.play_count = 0;
 
@@ -266,7 +267,24 @@ sample_get_wave_data (GByteArray * sample, struct job_control *control,
       error_print ("%s\n", sf_strerror (sndfile));
     }
 
-  total = sf_writef_short (sndfile, (gint16 *) sample->data, frames);
+  if ((sample_info->format & SF_FORMAT_SUBMASK) == SF_FORMAT_PCM_16)
+    {
+      total = sf_writef_short (sndfile, (gint16 *) sample->data, frames);
+    }
+  else if ((sample_info->format & SF_FORMAT_SUBMASK) == SF_FORMAT_FLOAT)
+    {
+      total = sf_writef_float (sndfile, (gfloat *) sample->data, frames);
+    }
+  else if ((sample_info->format & SF_FORMAT_SUBMASK) == SF_FORMAT_PCM_32)
+    {
+      total = sf_writef_int (sndfile, (gint32 *) sample->data, frames);
+    }
+  else
+    {
+      error_print ("Invalid sample format. Using short...\n");
+      total = sf_writef_short (sndfile, (gint16 *) sample->data, frames);
+    }
+
   sf_close (sndfile);
 
   if (total != frames)
@@ -280,21 +298,23 @@ sample_get_wave_data (GByteArray * sample, struct job_control *control,
 }
 
 gint
-sample_get_wav_from_array (GByteArray * sample, GByteArray * wave,
-			   struct job_control *control)
+sample_get_audio_file_data_from_array (GByteArray * sample, GByteArray * wave,
+				       struct job_control *control,
+				       guint32 format)
 {
   struct g_byte_array_io_data data;
   data.pos = 0;
   data.array = wave;
-  return sample_get_wave_data (sample, control, &data);
+  return sample_get_audio_file_data (sample, control, &data, format);
 }
 
 gint
-sample_save_from_array (const gchar * path, GByteArray * sample,
-			struct job_control *control)
+sample_save_to_file (const gchar * path, GByteArray * sample,
+		     struct job_control *control, guint32 format)
 {
   GByteArray *wave = g_byte_array_new ();
-  gint ret = sample_get_wav_from_array (sample, wave, control);
+  gint ret = sample_get_audio_file_data_from_array (sample, wave, control,
+						    format);
   if (!ret)
     {
       ret = save_file (path, wave, control);
@@ -304,12 +324,12 @@ sample_save_from_array (const gchar * path, GByteArray * sample,
 }
 
 static void
-audio_multichannel_to_mono (gshort * input, gshort * output, gint size,
-			    gint channels)
+audio_multichannel_to_mono_short (gshort * input, gshort * output, gint size,
+				  gint channels)
 {
-  int i, j, v;
+  gint32 i, j, v;
 
-  debug_print (2, "Converting to mono...\n");
+  debug_print (2, "Converting short values to mono...\n");
 
   for (i = 0; i < size; i++)
     {
@@ -318,38 +338,229 @@ audio_multichannel_to_mono (gshort * input, gshort * output, gint size,
 	{
 	  v += input[i * channels + j];
 	}
-      v /= channels;
+      v *= MULTICHANNEL_MIX_GAIN (channels);
       output[i] = v;
     }
 }
 
+static void
+audio_multichannel_to_mono_float (gfloat * input, gfloat * output, gint size,
+				  gint channels)
+{
+  gfloat v;
+  gint i, j;
+
+  debug_print (2, "Converting float values to mono...\n");
+
+  for (i = 0; i < size; i++)
+    {
+      v = 0;
+      for (j = 0; j < channels; j++)
+	{
+	  v += input[i * channels + j];
+	}
+      v *= MULTICHANNEL_MIX_GAIN (channels);
+      output[i] = v;
+    }
+}
+
+static void
+audio_multichannel_to_mono_int (gint32 * input, gint32 * output, gint size,
+				gint channels)
+{
+  gint32 v;
+  gint i, j;
+
+  debug_print (2, "Converting int values to mono...\n");
+
+  for (i = 0; i < size; i++)
+    {
+      v = 0;
+      for (j = 0; j < channels; j++)
+	{
+	  v += input[i * channels + j];
+	}
+      v *= MULTICHANNEL_MIX_GAIN (channels);
+      output[i] = v;
+    }
+}
+
+static void
+audio_mono_to_stereo_short (gshort * input, gshort * output, gint size)
+{
+  debug_print (2, "Converting short values to stereo...\n");
+
+  for (gint i = 0; i < size; i++, input++)
+    {
+      *output = *input;
+      output++;
+      *output = *input;
+      output++;
+    }
+}
+
+static void
+audio_mono_to_stereo_float (gfloat * input, gfloat * output, gint size)
+{
+  debug_print (2, "Converting float values to stereo...\n");
+
+  for (gint i = 0; i < size; i++, input++)
+    {
+      *output = *input;
+      output++;
+      *output = *input;
+      output++;
+    }
+}
+
+static void
+audio_mono_to_stereo_int (gint32 * input, gint32 * output, gint size)
+{
+  debug_print (2, "Converting int values to stereo...\n");
+
+  for (gint i = 0; i < size; i++, input++)
+    {
+      *output = *input;
+      output++;
+      *output = *input;
+      output++;
+    }
+}
+
+static void
+sample_set_sample_info (struct sample_info *sample_info, SNDFILE * sndfile,
+			SF_INFO * sf_info)
+{
+  struct SF_CHUNK_INFO chunk_info;
+  SF_CHUNK_ITERATOR *chunk_iter;
+  struct smpl_chunk_data smpl_chunk_data;
+  gboolean disable_loop = FALSE;
+
+  sample_info->channels = sf_info->channels;
+  sample_info->rate = sf_info->samplerate;
+  sample_info->frames = sf_info->frames;
+  sample_info->format = sf_info->format;
+
+  strcpy (chunk_info.id, SMPL_CHUNK_ID);
+  chunk_info.id_size = strlen (SMPL_CHUNK_ID);
+  chunk_iter = sf_get_chunk_iterator (sndfile, &chunk_info);
+
+  if (chunk_iter)
+    {
+      chunk_info.datalen = sizeof (struct smpl_chunk_data);
+      memset (&smpl_chunk_data, 0, chunk_info.datalen);
+      debug_print (2, "%s chunk found (%d B)\n", SMPL_CHUNK_ID,
+		   chunk_info.datalen);
+      chunk_info.data = &smpl_chunk_data;
+      sf_get_chunk_data (chunk_iter, &chunk_info);
+      sample_info->loop_start =
+	GUINT32_FROM_LE (smpl_chunk_data.sample_loop.start);
+      sample_info->loop_end =
+	GUINT32_FROM_LE (smpl_chunk_data.sample_loop.end);
+      sample_info->loop_type =
+	GUINT32_FROM_LE (smpl_chunk_data.sample_loop.type);
+      sample_info->midi_note =
+	GUINT32_FROM_LE (smpl_chunk_data.midi_unity_note);
+      if (sample_info->loop_start >= sample_info->frames)
+	{
+	  debug_print (2, "Bad loop start\n");
+	  disable_loop = TRUE;
+	}
+      if (sample_info->loop_end >= sample_info->frames)
+	{
+	  debug_print (2, "Bad loop end\n");
+	  disable_loop = TRUE;
+	}
+
+      while (chunk_iter)
+	{
+	  chunk_iter = sf_next_chunk_iterator (chunk_iter);
+	}
+    }
+  else
+    {
+      disable_loop = TRUE;
+      sample_info->midi_note = 0;
+    }
+  if (disable_loop)
+    {
+      sample_info->loop_start = sample_info->frames - 1;
+      sample_info->loop_end = sample_info->loop_start;
+      sample_info->loop_type = 0;
+    }
+
+  debug_print (2, "Loop start at %d, loop end at %d\n",
+	       sample_info->loop_start, sample_info->loop_end);
+}
+
+gint
+sample_load_sample_info (const gchar * path, struct sample_info *sample_info)
+{
+  SF_INFO sf_info;
+  SNDFILE *sndfile;
+  FILE *file;
+
+  memset (sample_info, 0, sizeof (struct sample_info));
+
+  file = fopen (path, "rb");
+  if (!file)
+    {
+      return -errno;
+    }
+
+  sndfile = sf_open_virtual (&FILE_IO, SFM_READ, &sf_info, file);
+  if (!sndfile)
+    {
+      error_print ("Error while reading %s: %s\n", path,
+		   sf_strerror (sndfile));
+      return -1;
+    }
+
+  sample_set_sample_info (sample_info, sndfile, &sf_info);
+
+  fclose (file);
+  return 0;
+}
+
+void
+sample_check_and_fix_loop_points (struct sample_info *sample_info)
+{
+  if (sample_info->loop_start >= sample_info->frames)
+    {
+      sample_info->loop_start = sample_info->frames - 1;
+    }
+  if (sample_info->loop_end >= sample_info->frames)
+    {
+      sample_info->loop_end = sample_info->frames - 1;
+    }
+}
+
 // If control->data is NULL, then a new struct sample_info * is created and control->data points to it.
 // In case of failure, if control->data is NULL is freed.
-// Franes is the amount of frames after resampling. This value is set before the loading has terminated.
 
 static gint
 sample_load_raw (void *data, SF_VIRTUAL_IO * sf_virtual_io,
 		 struct job_control *control, GByteArray * sample,
-		 const struct sample_params *sample_params, guint * frames,
-		 sample_load_cb cb, gpointer cb_data)
+		 struct sample_info *sample_info_dst, sample_load_cb cb,
+		 gpointer cb_data)
 {
   SF_INFO sf_info;
   SNDFILE *sndfile;
   SRC_DATA src_data;
   SRC_STATE *src_state;
-  struct SF_CHUNK_INFO chunk_info;
-  SF_CHUNK_ITERATOR *chunk_iter;
-  gint16 *buffer_input;
-  gint16 *buffer_input_multi;
-  gint16 *buffer_input_mono;
-  gint16 *buffer_s;
+  void *buffer_input;
+  void *buffer_input_multi;
+  void *buffer_input_mono;
+  void *buffer_input_stereo;
+  void *buffer_i;		//For gint16 or gint32
   gfloat *buffer_f;
-  gint err, resampled_buffer_len, f, frames_read, channels, samplerate;
+  void *buffer_output;
+  gint err, resampled_buffer_len, frames_read;
   gboolean active;
   gdouble ratio;
-  struct sample_info *sample_info;
-  struct smpl_chunk_data smpl_chunk_data;
-  gboolean disable_loop = FALSE;
+  struct sample_info *sample_info_src;
+  guint bytes_per_sample, bytes_per_frame;
+  guint32 f, actual_frames = 0;
 
   if (control)
     {
@@ -369,82 +580,44 @@ sample_load_raw (void *data, SF_VIRTUAL_IO * sf_virtual_io,
       return -1;
     }
 
-  sample_info = control->data;
-  if (!control->data)
+  sample_info_src = control->data;
+  if (!sample_info_src)
     {
-      sample_info = g_malloc (sizeof (struct sample_info));
+      sample_info_src = g_malloc (sizeof (struct sample_info));
     }
 
-  channels = sample_params->channels == 2 && sf_info.channels == 2 ? 2 : 1;
-  samplerate =
-    sample_params->samplerate ? sample_params->
-    samplerate : sf_info.samplerate;
-
   if (control)
     {
       g_mutex_lock (&control->mutex);
     }
-  sample_info->channels = sf_info.channels;
-  sample_info->samplerate = sf_info.samplerate;
-  sample_info->frames = sf_info.frames;
+
+  sample_info_dst->channels = sample_info_dst->channels ?
+    sample_info_dst->channels : sf_info.channels;
+  sample_info_dst->rate = sample_info_dst->rate ? sample_info_dst->rate :
+    sf_info.samplerate;
+  sample_info_dst->format =
+    sample_info_dst->format ? sample_info_dst->format : SF_FORMAT_PCM_16;
+
+  if (sample_info_dst->format != SF_FORMAT_PCM_16 &&
+      sample_info_dst->format != SF_FORMAT_PCM_32 &&
+      sample_info_dst->format != SF_FORMAT_FLOAT)
+    {
+      error_print ("Invalid sample format. Using short...\n");
+      sample_info_dst->format = SF_FORMAT_PCM_16;
+    }
+
+  bytes_per_frame = SAMPLE_INFO_FRAME_SIZE (sample_info_dst);
+  bytes_per_sample = SAMPLE_SIZE (sample_info_dst->format);
+
+  sample_set_sample_info (sample_info_src, sndfile, &sf_info);
+  sample_info_dst->midi_note = sample_info_src->midi_note;
+  sample_info_dst->loop_type = sample_info_src->loop_type;
+
   if (control)
     {
       g_mutex_unlock (&control->mutex);
     }
 
-  switch (sf_info.format & SF_FORMAT_SUBMASK)
-    {
-    case SF_FORMAT_PCM_S8:
-      sample_info->bitdepth = 8;
-      break;
-    case SF_FORMAT_PCM_16:
-      sample_info->bitdepth = 16;
-      break;
-    case SF_FORMAT_PCM_24:
-      sample_info->bitdepth = 24;
-      break;
-    case SF_FORMAT_PCM_32:
-      sample_info->bitdepth = 32;
-      break;
-    default:
-      sample_info->bitdepth = 0;
-    }
-
-  strcpy (chunk_info.id, SMPL_CHUNK_ID);
-  chunk_info.id_size = strlen (SMPL_CHUNK_ID);
-  chunk_iter = sf_get_chunk_iterator (sndfile, &chunk_info);
-
-  if (chunk_iter)
-    {
-      chunk_info.datalen = sizeof (struct smpl_chunk_data);
-      memset (&smpl_chunk_data, 0, chunk_info.datalen);
-      chunk_info.data = &smpl_chunk_data;
-      sf_get_chunk_data (chunk_iter, &chunk_info);
-      sample_info->loopstart = le32toh (smpl_chunk_data.sample_loop.start);
-      sample_info->loopend = le32toh (smpl_chunk_data.sample_loop.end);
-      sample_info->looptype = le32toh (smpl_chunk_data.sample_loop.type);
-      if (sample_info->loopstart >= sf_info.frames)
-	{
-	  debug_print (2, "Bad loop start\n");
-	  disable_loop = TRUE;
-	}
-      if (sample_info->loopend >= sf_info.frames)
-	{
-	  debug_print (2, "Bad loop end\n");
-	  disable_loop = TRUE;
-	}
-    }
-  else
-    {
-      disable_loop = TRUE;
-    }
-  if (disable_loop)
-    {
-      sample_info->loopstart = sf_info.frames - 1;
-      sample_info->loopend = sample_info->loopstart;
-      sample_info->looptype = 0;
-    }
-
   //Set scale factor. See http://www.mega-nerd.com/libsndfile/api.html#note2
   if ((sf_info.format & SF_FORMAT_FLOAT) == SF_FORMAT_FLOAT ||
       (sf_info.format & SF_FORMAT_DOUBLE) == SF_FORMAT_DOUBLE)
@@ -454,39 +627,54 @@ sample_load_raw (void *data, SF_VIRTUAL_IO * sf_virtual_io,
       sf_command (sndfile, SFC_SET_SCALE_FLOAT_INT_READ, NULL, SF_TRUE);
     }
 
-  buffer_input_multi =
-    malloc (LOAD_BUFFER_LEN * sf_info.channels * sizeof (gint16));
-  buffer_input_mono = malloc (LOAD_BUFFER_LEN * sizeof (gint16));
+  buffer_input_multi = g_malloc (LOAD_BUFFER_LEN *
+				 FRAME_SIZE (sample_info_src->channels,
+					     sample_info_dst->format));
+  buffer_input_mono = g_malloc (LOAD_BUFFER_LEN * bytes_per_sample);
+  buffer_input_stereo = g_malloc (LOAD_BUFFER_LEN * 2 * bytes_per_sample);
 
-  buffer_f = malloc (LOAD_BUFFER_LEN * channels * sizeof (gfloat));
-  src_data.data_in = buffer_f;
-  ratio = samplerate / (double) sf_info.samplerate;
+  ratio = sample_info_dst->rate / (double) sample_info_src->rate;
   src_data.src_ratio = ratio;
 
   src_data.output_frames = ceil (LOAD_BUFFER_LEN * src_data.src_ratio);
-  resampled_buffer_len = src_data.output_frames * channels;
-  buffer_s = malloc (resampled_buffer_len * sizeof (gint16));
-  src_data.data_out = malloc (resampled_buffer_len * sizeof (gfloat));
+  resampled_buffer_len = src_data.output_frames * sample_info_dst->channels;
+  buffer_i = g_malloc (resampled_buffer_len * bytes_per_sample);
+  src_data.data_out = g_malloc (resampled_buffer_len * sizeof (gfloat));
+  if (sample_info_dst->format == SF_FORMAT_PCM_16 ||
+      sample_info_dst->format == SF_FORMAT_PCM_32)
+    {
+      buffer_f = g_malloc (LOAD_BUFFER_LEN * sample_info_dst->channels *
+			   sizeof (gfloat));
+      src_data.data_in = buffer_f;
+      buffer_output = buffer_i;
+    }
+  else
+    {
+      buffer_f = NULL;
+      buffer_output = src_data.data_out;
+    }
 
-  src_state = src_new (SRC_SINC_BEST_QUALITY, channels, &err);
+  src_state = src_new (SRC_SINC_BEST_QUALITY, sample_info_dst->channels,
+		       &err);
   if (err)
     {
       goto cleanup;
     }
 
-  *frames = sf_info.frames * src_data.src_ratio;
-
-  if (samplerate != sf_info.samplerate)
+  if (control)
     {
-      debug_print (2, "Loop start at %d, loop end at %d before resampling\n",
-		   sample_info->loopstart, sample_info->loopend);
-      sample_info->loopstart = round (sample_info->loopstart * ratio);
-      sample_info->loopend = round (sample_info->loopend * ratio);
-      debug_print (2, "Loop start at %d, loop end at %d after resampling\n",
-		   sample_info->loopstart, sample_info->loopend);
+      g_mutex_lock (&control->mutex);
+    }
+  sample_info_dst->frames = floor (sample_info_src->frames * ratio);	//Lower bound estimation. The actual amount is updated later.
+  sample_info_dst->loop_start = round (sample_info_src->loop_start * ratio);
+  sample_info_dst->loop_end = round (sample_info_src->loop_end * ratio);
+  sample_check_and_fix_loop_points (sample_info_dst);
+  if (control)
+    {
+      g_mutex_unlock (&control->mutex);
     }
 
-  debug_print (2, "Loading sample (%" PRId64 " frames)...\n", sf_info.frames);
+  debug_print (2, "Loading sample (%d frames)...\n", sample_info_src->frames);
 
   if (control)
     {
@@ -500,32 +688,94 @@ sample_load_raw (void *data, SF_VIRTUAL_IO * sf_virtual_io,
     }
 
   f = 0;
-  while (f < sf_info.frames && active)
+  while (f < sample_info_src->frames && active)
     {
-      debug_print (2, "Loading %d channels buffer...\n", channels);
-      frames_read = sf_readf_short (sndfile, buffer_input_multi,
-				    LOAD_BUFFER_LEN);
+      debug_print (2, "Loading %d channels buffer...\n",
+		   sample_info_dst->channels);
+
+      if (sample_info_dst->format == SF_FORMAT_FLOAT)
+	{
+	  frames_read = sf_readf_float (sndfile,
+					(gfloat *) buffer_input_multi,
+					LOAD_BUFFER_LEN);
+	}
+      else if (sample_info_dst->format == SF_FORMAT_PCM_32)
+	{
+	  frames_read = sf_readf_int (sndfile,
+				      (gint32 *) buffer_input_multi,
+				      LOAD_BUFFER_LEN);
+	}
+      else
+	{
+	  frames_read = sf_readf_short (sndfile,
+					(gint16 *) buffer_input_multi,
+					LOAD_BUFFER_LEN);
+	}
       f += frames_read;
 
-      if (channels == sf_info.channels)	// 1 <= channels <= 2
+      if (sample_info_dst->channels == sample_info_src->channels)
 	{
 	  buffer_input = buffer_input_multi;
 	}
       else
 	{
-	  audio_multichannel_to_mono (buffer_input_multi, buffer_input_mono,
-				      frames_read, sf_info.channels);
-	  buffer_input = buffer_input_mono;
+	  if (sample_info_dst->format == SF_FORMAT_FLOAT)
+	    {
+	      audio_multichannel_to_mono_float (buffer_input_multi,
+						buffer_input_mono,
+						frames_read,
+						sample_info_src->channels);
+	    }
+	  else if (sample_info_dst->format == SF_FORMAT_PCM_32)
+	    {
+	      audio_multichannel_to_mono_int (buffer_input_multi,
+					      buffer_input_mono,
+					      frames_read,
+					      sample_info_src->channels);
+	    }
+	  else
+	    {
+	      audio_multichannel_to_mono_short (buffer_input_multi,
+						buffer_input_mono,
+						frames_read,
+						sample_info_src->channels);
+	    }
+	  if (sample_info_dst->channels == 1)
+	    {
+	      buffer_input = buffer_input_mono;
+	    }
+	  else
+	    {
+	      if (sample_info_dst->format == SF_FORMAT_FLOAT)
+		{
+		  audio_mono_to_stereo_float (buffer_input_mono,
+					      buffer_input_stereo,
+					      frames_read);
+		}
+	      else if (sample_info_dst->format == SF_FORMAT_PCM_32)
+		{
+		  audio_mono_to_stereo_int (buffer_input_mono,
+					    buffer_input_stereo, frames_read);
+		}
+	      else
+		{
+		  audio_mono_to_stereo_short (buffer_input_mono,
+					      buffer_input_stereo,
+					      frames_read);
+		}
+	      buffer_input = buffer_input_stereo;
+	    }
 	}
 
-      if (samplerate == sf_info.samplerate)
+      if (sample_info_dst->rate == sample_info_src->rate)
 	{
 	  if (control)
 	    {
 	      g_mutex_lock (&control->mutex);
 	    }
 	  g_byte_array_append (sample, (guint8 *) buffer_input,
-			       frames_read << channels);
+			       frames_read * bytes_per_frame);
+	  actual_frames += frames_read;
 	  if (control)
 	    {
 	      g_mutex_unlock (&control->mutex);
@@ -536,10 +786,23 @@ sample_load_raw (void *data, SF_VIRTUAL_IO * sf_virtual_io,
 	  src_data.end_of_input = frames_read < LOAD_BUFFER_LEN ? SF_TRUE : 0;
 	  src_data.input_frames = frames_read;
 
-	  src_short_to_float_array (buffer_input, buffer_f,
-				    frames_read * channels);
+	  if (sample_info_dst->format == SF_FORMAT_FLOAT)
+	    {
+	      src_data.data_in = buffer_input;
+	    }
+	  else if (sample_info_dst->format == SF_FORMAT_PCM_32)
+	    {
+	      src_int_to_float_array (buffer_input, buffer_f, frames_read *
+				      sample_info_dst->channels);
+	    }
+	  else
+	    {
+	      src_short_to_float_array (buffer_input, buffer_f, frames_read *
+					sample_info_dst->channels);
+	    }
+
 	  debug_print (2, "Resampling %d channels with ratio %f...\n",
-		       channels, src_data.src_ratio);
+		       sample_info_dst->channels, src_data.src_ratio);
 	  err = src_process (src_state, &src_data);
 	  if (err)
 	    {
@@ -548,14 +811,27 @@ sample_load_raw (void *data, SF_VIRTUAL_IO * sf_virtual_io,
 			   src_strerror (err));
 	      break;
 	    }
-	  src_float_to_short_array (src_data.data_out, buffer_s,
-				    src_data.output_frames_gen * channels);
+
 	  if (control)
 	    {
 	      g_mutex_lock (&control->mutex);
 	    }
-	  g_byte_array_append (sample, (guint8 *) buffer_s,
-			       src_data.output_frames_gen << channels);
+
+	  if (sample_info_dst->format == SF_FORMAT_PCM_32)
+	    {
+	      src_float_to_int_array (src_data.data_out, buffer_i,
+				      src_data.output_frames_gen *
+				      sample_info_dst->channels);
+	    }
+	  if (sample_info_dst->format == SF_FORMAT_PCM_16)
+	    {
+	      src_float_to_short_array (src_data.data_out, buffer_i,
+					src_data.output_frames_gen *
+					sample_info_dst->channels);
+	    }
+	  g_byte_array_append (sample, (guint8 *) buffer_output,
+			       src_data.output_frames_gen * bytes_per_frame);
+	  actual_frames += src_data.output_frames_gen;
 	  if (control)
 	    {
 	      g_mutex_unlock (&control->mutex);
@@ -565,7 +841,7 @@ sample_load_raw (void *data, SF_VIRTUAL_IO * sf_virtual_io,
       if (control)
 	{
 	  g_mutex_lock (&control->mutex);
-	  cb (control, f * 1.0 / sf_info.frames, cb_data);
+	  cb (control, f * 1.0 / sample_info_src->frames, cb_data);
 	  active = control->active;
 	  g_mutex_unlock (&control->mutex);
 	}
@@ -576,25 +852,24 @@ sample_load_raw (void *data, SF_VIRTUAL_IO * sf_virtual_io,
   if (control)
     {
       g_mutex_lock (&control->mutex);
-
-      if (control->active)
-	{
-	  cb (control, 1.0, cb_data);
-	}
-      else
+      if (!control->active)
 	{
 	  g_byte_array_set_size (sample, 0);
 	}
-
       g_mutex_unlock (&control->mutex);
     }
 
 cleanup:
-  free (buffer_input_multi);
-  free (buffer_input_mono);
-  free (buffer_s);
-  free (buffer_f);
-  free (src_data.data_out);
+  g_free (buffer_input_multi);
+  g_free (buffer_input_mono);
+  g_free (buffer_input_stereo);
+  g_free (buffer_i);
+  if (sample_info_dst->format == SF_FORMAT_PCM_16 ||
+      sample_info_dst->format == SF_FORMAT_PCM_32)
+    {
+      g_free (buffer_f);
+    }
+  g_free (src_data.data_out);
 
   sf_close (sndfile);
 
@@ -602,17 +877,29 @@ cleanup:
     {
       if (!control->data)
 	{
-	  control->data = sample_info;
+	  control->data = sample_info_src;
+	}
+      if (control)
+	{
+	  g_mutex_lock (&control->mutex);
 	}
       // This removes the additional samples added by the resampler due to rounding.
-      g_byte_array_set_size (sample, *frames << channels);
+      sample_info_dst->frames = actual_frames;
+      sample_check_and_fix_loop_points (sample_info_dst);
+      g_byte_array_set_size (sample,
+			     sample_info_dst->frames * bytes_per_frame);
+      if (control)
+	{
+	  cb (control, 1.0, cb_data);
+	  g_mutex_unlock (&control->mutex);
+	}
       return 0;
     }
   else
     {
       if (!control->data)
 	{
-	  g_free (sample_info);
+	  g_free (sample_info_src);
 	}
       return -1;
     }
@@ -621,31 +908,29 @@ cleanup:
 gint
 sample_load_from_array (GByteArray * wave, GByteArray * sample,
 			struct job_control *control,
-			const struct sample_params *sample_params,
-			guint * frames)
+			struct sample_info *sample_info_dst)
 {
   struct g_byte_array_io_data data;
   data.pos = 0;
   data.array = wave;
   return sample_load_raw (&data, &G_BYTE_ARRAY_IO, control, sample,
-			  sample_params, frames,
-			  set_job_control_progress_no_sync, NULL);
+			  sample_info_dst, set_job_control_progress_no_sync,
+			  NULL);
 }
 
 gint
 sample_load_from_file_with_cb (const gchar * path, GByteArray * sample,
 			       struct job_control *control,
-			       const struct sample_params *sample_params,
-			       guint * frames, sample_load_cb cb,
-			       gpointer cb_data)
+			       struct sample_info *sample_info_dst,
+			       sample_load_cb cb, gpointer cb_data)
 {
   FILE *file = fopen (path, "rb");
   if (!file)
     {
       return -errno;
     }
-  gint err = sample_load_raw (file, &FILE_IO, control, sample, sample_params,
-			      frames, cb, cb_data);
+  gint err = sample_load_raw (file, &FILE_IO, control, sample,
+			      sample_info_dst, cb, cb_data);
   fclose (file);
   return err;
 }
@@ -653,39 +938,64 @@ sample_load_from_file_with_cb (const gchar * path, GByteArray * sample,
 gint
 sample_load_from_file (const gchar * path, GByteArray * sample,
 		       struct job_control *control,
-		       const struct sample_params *sample_params,
-		       guint * frames)
+		       struct sample_info *sample_info_dst)
 {
   return sample_load_from_file_with_cb (path, sample, control,
-					sample_params, frames,
+					sample_info_dst,
 					set_job_control_progress_no_sync,
 					NULL);
 }
 
-static gboolean
-sample_is_mp3_supported ()
-{
-#if defined(__linux__)
-  static char buffer[LABEL_MAX];
-  sf_command (NULL, SFC_GET_LIB_VERSION, buffer, LABEL_MAX);
-  debug_print (1, "libsndfile version: %s...\n", buffer);
-  if (strverscmp (buffer, "libsndfile-1.1.0") >= 0)
-    {
-      return TRUE;
-    }
-
-  return FALSE;
-#else
-  return TRUE;
-#endif
-}
-
 const gchar **
 sample_get_sample_extensions ()
 {
-  if (sample_is_mp3_supported ())
-    {
-      return ELEKTROID_AUDIO_LOCAL_EXTS_MP3;
-    }
   return ELEKTROID_AUDIO_LOCAL_EXTS;
 }
+
+const gchar *
+sample_get_format (struct sample_info *sample_info)
+{
+  switch (sample_info->format & SF_FORMAT_TYPEMASK)
+    {
+    case SF_FORMAT_WAV:
+      return "WAV";
+    case SF_FORMAT_AIFF:
+      return "AIFF";
+    case SF_FORMAT_AU:
+      return "Au";
+    case SF_FORMAT_FLAC:
+      return "FLAC";
+    case SF_FORMAT_OGG:
+      return "Ogg";
+#if !defined(__linux__) || HAVE_SNDFILE_MP3
+    case SF_FORMAT_MPEG:
+      return "MPEG";
+#endif
+    default:
+      return "?";
+    }
+}
+
+const gchar *
+sample_get_subtype (struct sample_info *sample_info)
+{
+  switch (sample_info->format & SF_FORMAT_SUBMASK)
+    {
+    case SF_FORMAT_PCM_S8:
+      return "s8";
+    case SF_FORMAT_PCM_16:
+      return "s16";
+    case SF_FORMAT_PCM_24:
+      return "s24";
+    case SF_FORMAT_PCM_32:
+      return "s32";
+    case SF_FORMAT_PCM_U8:
+      return "u8";
+    case SF_FORMAT_FLOAT:
+      return "f32";
+    case SF_FORMAT_DOUBLE:
+      return "f64";
+    default:
+      return "?";
+    }
+}
