@@ -79,14 +79,12 @@ debug_get_hex_data (gint level, guint8 * data, guint len)
 	}
     }
   size = bytes_shown * 3 + extra;
-  str = malloc (sizeof (char) * size);
-
-  if (!len)
+  if (!size)
     {
-      *str = 0;
-      return str;
+      return NULL;
     }
 
+  str = g_malloc (sizeof (char) * size);
   b = data;
   next = str;
 
@@ -212,14 +210,25 @@ get_filename (guint32 fs_options, struct item *item)
 {
   if (fs_options & FS_OPTION_ID_AS_FILENAME && item->type == ELEKTROID_FILE)
     {
-      gchar *id = malloc (LABEL_MAX);
-      snprintf (id, LABEL_MAX, "%d", item->id);
+      gchar *dir = g_path_get_dirname (item->name);
+      gboolean no_dir = strcmp (dir, ".") == 0;
+      gchar *id = g_malloc (LABEL_MAX);
+      if (no_dir)
+	{
+	  snprintf (id, LABEL_MAX, "%d", item->id);
+	}
+      else
+	{
+	  // If FS_OPTION_ID_AS_FILENAME is used, then the path separator is always '/'.
+	  snprintf (id, LABEL_MAX, "%s/%d", dir, item->id);
+	}
+      g_free (dir);
       return id;
     }
   return strdup (item->name);
 }
 
-guint
+gint
 next_item_iterator (struct item_iterator *iter)
 {
   return iter->next (iter);
@@ -320,7 +329,7 @@ save_file (const gchar * path, GByteArray * array,
 gchar *
 get_human_size (gint64 size, gboolean with_space)
 {
-  gchar *label = malloc (LABEL_MAX);
+  gchar *label = g_malloc (LABEL_MAX);
   gchar *space = with_space ? " " : "";
 
   if (size < 0)
@@ -333,16 +342,16 @@ get_human_size (gint64 size, gboolean with_space)
     }
   else if (size < KIB * KIB)
     {
-      snprintf (label, LABEL_MAX, "%.2f%sKiB", size / (double) KIB, space);
+      snprintf (label, LABEL_MAX, "%.4g%sKiB", size / (double) KIB, space);
     }
   else if (size < KIB * KIB * KIB)
     {
-      snprintf (label, LABEL_MAX, "%.2f%sMiB", size / (double) (KIB * KIB),
+      snprintf (label, LABEL_MAX, "%.4g%sMiB", size / (double) (KIB * KIB),
 		space);
     }
   else
     {
-      snprintf (label, LABEL_MAX, "%.2f%sGiB",
+      snprintf (label, LABEL_MAX, "%.4g%sGiB",
 		size / (double) (KIB * KIB * KIB), space);
     }
 
@@ -389,10 +398,10 @@ set_job_control_progress_no_sync (struct job_control *control, gdouble p,
 }
 
 gboolean
-file_matches_extensions (const gchar * name, gchar ** extensions)
+file_matches_extensions (const gchar * name, const gchar ** extensions)
 {
   const gchar *extension;
-  gchar **e = extensions;
+  const gchar **e = extensions;
 
   if (!e)
     {
@@ -418,7 +427,8 @@ file_matches_extensions (const gchar * name, gchar ** extensions)
 }
 
 gboolean
-iter_matches_extensions (struct item_iterator *iter, gchar ** extensions)
+iter_is_dir_or_matches_extensions (struct item_iterator *iter,
+				   const gchar ** extensions)
 {
   if (iter->item.type == ELEKTROID_DIR)
     {
